#include <stdint.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netdb.h>

#include "constants.h"

#ifndef _TYPES_H
#define	_TYPES_H

typedef struct
{
    int32_t Longitude;                                  // in 1E-7 deg
    int32_t Latitude;                                   // in 1E-7 deg
    int32_t Altitude;                                   // in mm
    uint8_t Status;                                      // validity of data
} __attribute__ ( ( packed ) ) GPS_Pos_t;

typedef struct
{
    GPS_Pos_t Position;                                  // position of the client
    uint8_t MAC[6];                                      // mac address of the client
} __attribute__ ( ( packed ) ) Submitted_Pos_t;

typedef struct
{
    uint16_t Distance;                                   // distance to target in dm
    int16_t Bearing;                                    // course to target in deg
} __attribute__ ( ( packed ) ) GPS_PosDev_t;

typedef struct
{
    uint8_t Version;                                     // version of the data structure
    GPS_Pos_t CurrentPosition;                      // see ubx.h for details
    GPS_Pos_t TargetPosition;
    GPS_PosDev_t TargetPositionDeviation;
    GPS_Pos_t HomePosition;
    GPS_PosDev_t HomePositionDeviation;
    uint8_t  WaypointIndex;                              // index of current waypoints running from 0 to WaypointNumber-1
    uint8_t  WaypointNumber;                             // number of stored waypoints
    uint8_t  SatsInUse;                                  // number of satellites used for position solution
    int16_t Altimeter;                                  // hight according to air pressure
    int16_t Variometer;                                 // climb(+) and sink(-) rate
    uint16_t FlyingTime;                                 // in seconds
    uint8_t  UBat;                                       // Battery Voltage in 0.1 Volts
    uint16_t GroundSpeed;                                // speed over ground in cm/s (2D)
    int16_t Heading;                                    // current flight direction in ° as angle to north
    int16_t CompassHeading;                             // current compass value in °
    int8_t  AngleNick;                                  // current Nick angle in 1°
    int8_t  AngleRoll;                                  // current Rick angle in 1°
    uint8_t  RC_Quality;                                 // RC_Quality
    uint8_t  FCFlags;                                    // Flags from FC
    uint8_t  NCFlags;                                    // Flags from NC
    uint8_t  Errorcode;                                  // 0 --> okay
    uint8_t  OperatingRadius;                            // current operation radius around the Home Position in m
    int16_t TopSpeed;                                   // velocity in vertical direction in cm/s
    uint8_t  TargetHoldTime;                             // time in s to stay at the given target, counts down to 0 if target has been reached
    uint8_t  RC_RSSI;                                    // Receiver signal strength (since version 2 added)
    int16_t SetpointAltitude;                           // setpoint for altitude
    uint8_t  Gas;                                        // for future use
    uint16_t Current;                                    // actual current in 0.1A steps
    uint16_t UsedCapacity;                               // used capacity in mAh
} __attribute__ ( ( packed ) ) NaviData_t;


typedef struct
{
    GPS_Pos_t Position;       			    // Lat/Lon/Alt
    uint8_t Flags;                                       // Status Flags
    uint8_t NumOfSats;                                   // number of satelites
    uint8_t SatFix;                                      // type of satfix
    uint32_t Position_Accuracy;			    // in cm 3d position accuracy
    int32_t Speed_North;                                // in cm/s
    int32_t Speed_East;                                 // in cm/s
    int32_t Speed_Top;                                  // in cm/s
    uint32_t Speed_Ground;                               // 2D ground speed in cm/s
    int32_t Heading;                                    // 1e-05 deg  Heading 2-D (current flight direction)
    uint32_t Speed_Accuracy;                             // in cm/s 3d velocity accuracy
    uint8_t Status;					    // status of data
} __attribute__ ( ( packed ) ) GPS_Data_t;


typedef struct
{
    GPS_Pos_t Position;                             // the gps position of the waypoint, see ubx.h for details
    int16_t Heading;                                    // orientation, future implementation
    uint8_t  ToleranceRadius;                            // in meters, if the MK is within that range around the target, then the next target is triggered
    uint8_t  HoldTime;                                   // in seconds, if the was once in the tolerance area around a WP, this time defines the delay before the next WP is triggered
    uint8_t  Event_Flag;                                 // future implementation
    uint8_t  Index;              			    // to indentify different waypoints, workaround for bad communications PC <-> NC
    uint8_t  Reserve[11];                                // reserve
} __attribute__ ( ( packed ) ) Waypoint_t;

typedef struct
{
    int16_t Nick;
    int16_t Roll;
    uint8_t Param1;
    uint8_t Param2;
    uint8_t CalcState;
    uint8_t Orientation;
} __attribute__ ( ( packed ) ) Attribute_t;


typedef struct
{
    uint8_t Scenario;
    uint8_t State;
    uint8_t Positioning;
    uint8_t Searching;
    GPS_Pos_t Position;
    Submitted_Pos_t Client1;
    Submitted_Pos_t Client2;
    char Hostname[MAXHOSTNAMELENGTH];
    char IPv4[MAXIPV4LENGTH];
    uint8_t MAC[6];
} __attribute__ ( ( packed ) ) Status_t;

typedef struct
{
    GPS_Pos_t Position;
    int16_t Height;                                        // height according to air pressure
    uint8_t Battery;                                        // Battery Voltage in 0.1 Volts
    int16_t Heading;                                        // current flight direction in ° as angle to north
    uint16_t Speed;                                         // speed over ground in cm/s (2D)
    Status_t Status;
} __attribute__ ( ( packed ) ) Notification_t;

typedef struct
{
    uint8_t Subscription;
    struct sockaddr_storage Addr;
} __attribute__ ( ( packed ) ) NotificationSubscription_t;

typedef struct
{
    uint8_t MAC1[6];                                      // mac address of the client1
    uint8_t MAC2[6];  					// mac address of the client2
} __attribute__ ( ( packed ) ) AllowedClients_t;

typedef struct
{
    uint8_t Scenario;
    uint8_t Searching;
    uint8_t Positioning;
    AllowedClients_t AllowedClients;
} __attribute__ ( ( packed ) ) StartConfig_t;


#endif	/* _TYPES_H */
